#include "BootLoader/boot_loader.hpp"
#include <PSX/System/IOPorts/dMa_io.hpp>

// 2x For setup timing
#include <PSX/Timer/high_res_timer.hpp>
#include <stdio.h>

namespace JabyEngine {
    namespace boot {
        namespace Start {    
            static void enable_DMA() {
                const auto dpcr = DMA_IO::DPCR_t(DMA_IO::DPCR).set(DMA_IO::DPCR_t::SPUEnable).set(DMA_IO::DPCR_t::GPUEnable);
                DMA_IO::DPCR = dpcr;
            }

            JabyEngine::NextRoutine setup() {
                enable_DMA();

                SPU::stop_voices();
                
                CD::setup();
                Timer::setup();

                const auto start = HighResTime::get_time_stamp();
                printf("Start...\n");
                GPU::setup();
                GPU::display_logo();
                const auto end   = HighResTime::get_time_stamp();
                printf("GPU setup took %ims %ius\n", start.milliseconds_to(end), start.microseconds_to(end));
                
                //Pause??

                SPU::setup();
                return BootFile::setup();
            }
        }
    }
}